use std::borrow::Cow;

use yew::prelude::*;
use yew::virtual_dom::{VNode, VTag};

use pwt_macros::widget;

use crate::props::WidgetBuilder;

use super::SvgLength;

/// SVG `<circle>` element.
#[widget(pwt=crate, @element, @svg)]
#[derive(Properties, Clone, PartialEq)]
pub struct Circle {
    #[prop_or_default]
    children: Option<Vec<VNode>>,
}

impl Default for Circle {
    fn default() -> Self {
        Self::new()
    }
}

impl Circle {
    /// Create a new instance.
    pub fn new() -> Self {
        yew::props!(Self {})
    }

    /// Builder style method to set the circle center position.
    pub fn position(mut self, cx: impl Into<SvgLength>, cy: impl Into<SvgLength>) -> Self {
        self.set_position(cx, cy);
        self
    }

    /// Method to set the circle center position.
    pub fn set_position(&mut self, cx: impl Into<SvgLength>, cy: impl Into<SvgLength>) {
        self.set_attribute("cx", cx.into());
        self.set_attribute("cy", cy.into());
    }

    /// Builder style method to set the circle center x position.
    pub fn cx(mut self, cx: impl Into<SvgLength>) -> Self {
        self.set_cx(cx);
        self
    }

    /// Method to set the circle center x position.
    pub fn set_cx(&mut self, cx: impl Into<SvgLength>) {
        self.set_attribute("cx", cx.into());
    }

    /// Builder style method to set the circle center y position.
    pub fn cy(mut self, cy: impl Into<SvgLength>) -> Self {
        self.set_cy(cy);
        self
    }

    /// Method to set the circle center y position.
    pub fn set_cy(&mut self, cy: impl Into<SvgLength>) {
        self.set_attribute("cy", cy.into());
    }

    /// Builder style method to set the circle radius.
    pub fn r(mut self, r: impl Into<SvgLength>) -> Self {
        self.set_r(r);
        self
    }

    /// Method to set the circle radius.
    pub fn set_r(&mut self, r: impl Into<SvgLength>) {
        self.set_attribute("r", r.into());
    }

    impl_svg_animation_attributes!();
    impl_svg_presentation_attributes!();
}

impl From<Circle> for VTag {
    fn from(val: Circle) -> Self {
        val.std_props.into_vtag(
            Cow::Borrowed("circle"),
            None::<&str>,
            Some(val.listeners),
            val.children,
        )
    }
}
